#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::ProductBatchProcessHandler
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::ProductBatchProcessHandler;
use base Test::Class;

use strict;
use warnings;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    DropAllSearchIndexes
    GetOrCreateSearchIndex
);
use File::Basename;
use Test::MockObject;
use Test::More;

my $ROOTALIAS    = 'Test';
my $ROOTPATH     = JoinObjectPath( [ '', $ROOTALIAS ] );
my $TESTSHOPPATH = JoinObjectPath( [ '', $ROOTALIAS, 'TestShopRemoteSearch' ] );

sub init : Test(startup) {
    my $self = shift;
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};

    # forces remote communication to be off.
    DropAllSearchIndexes( LoadObjectByPath($TESTSHOPPATH) ) if ( ExistsObjectByPath($TESTSHOPPATH) );
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
}

sub setUp : Test(setup) {
    my $self = shift;
    $self->{Shop}        = LoadObjectByPath($TESTSHOPPATH);
    GetOrCreateSearchIndex( $self->shop );
}

sub runWithDefaultConstructor : Test(no_plan) {
    my $self = shift;
    my $Handler   = DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler->new();
    my $Mock = $self->_mockRemoteSearch();

    # mock remote adapter.
    ## no critic (ProtectPrivateVars)
    local *DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler::_remoteSearch = sub {
        return $Mock;
    };

    $Handler->addItem($_) foreach (@{$self->shop->get('Products')});
    $Handler->processItemList();
    cmp_ok( $Handler->countItems(), '==', 0, 'remaining buffer empty' );
    is($Mock->{'calls'}, 1, 'mock called once.');
}

sub runWithBatchSize : Test(no_plan) {
    my $self = shift;

    my $Handler   = DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler->new({
            BatchSize => 3,
    });
    my $Mock = $self->_mockRemoteSearch();

    # mock remote adapter.
    ## no critic (ProtectPrivateVars)
    local *DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler::_remoteSearch = sub {
        return $Mock;
    };

    my $aProducts = $self->shop->get('Products');
    $Handler->addItem($_) foreach (@{$aProducts});
    is($Mock->{'calls'}, 2, 'mock called twice.');
    is($Handler->countItems(), 1, 'remaining buffer filled with 1 item');
}

sub _mockUpdate {
    my $self = shift;
    my ($items) = @_;
    cmp_ok(scalar @$items, '>', 0, 'passed an array of ' . scalar @$items . ' items.') or diag($items);
    $self->{'calls'}++;
    return;
}

sub _mockRemoteSearch {
    my $self = shift;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::RemoteSearch::API::Object::SearchIndex');
    my %mockMethods = (updateItems => \&_mockUpdate );
    $mockRemote->fake_module('DE_EPAGES::RemoteSearch::API::Object::SearchIndex', %mockMethods);
    while (my ($method, $handler) = each %mockMethods) {
        $mockRemote->mock($method, $handler);
    }

    return $mockRemote;
}

sub shop {
    return shift->{Shop};
}

####### Main ############
RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        eval {
                            Test::Class->runtests;
                        };
                        if (ExistsError()) {
                            fail(GetError->longMessage);
                        }
                    }
                );
            }
        );
    }
);