#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::SearchIndex
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::SearchIndex;
use base Test::Class;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::RemoteSearch::API::IndexStatus qw (
    IndexStatusID
    IndexStatusString
    INDEXSTATUS_COMPLETE
    INDEXSTATUS_ERROR
    INDEXSTATUS_PENDING
    INDEXSTATUS_UNITIALIZED
);
use DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    DropAllSearchIndexes
    GetOrCreateSearchIndex
);
use File::Basename;
use Test::MockObject;
use Test::More;

use strict;

my $ROOTALIAS    = 'Test';
my $ROOTPATH     = JoinObjectPath( [ '', $ROOTALIAS ] );
my $TESTSHOPPATH = JoinObjectPath( [ '', $ROOTALIAS, 'TestShopRemoteSearch' ] );

sub init : Test(startup) {
    my $self = shift;
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};

    # forces remote communication to be off.
    DropAllSearchIndexes( LoadObjectByPath($TESTSHOPPATH) ) if ( ExistsObjectByPath($TESTSHOPPATH) );
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
}

sub setUp : Test(setup) {
    my $self = shift;
    $self->{Shop}        = LoadObjectByPath($TESTSHOPPATH);
    $self->{SearchIndex} = GetOrCreateSearchIndex( $self->shop );
}

sub TestClassName : Test {
    my $self = shift;
    is( $self->searchIndex->className, 'SearchIndex', 'className ok' );
}

sub TestIncludeTemplates : Test {
    my $self = shift;
    ok(scalar @{$self->shop->get('IncludeTemplates')}, 'template types ok');
}

sub TestPublicSuggestURLUsingProxy : Test(no_plan) {
    my $self = shift;

    # mock configuration.
    ## no critic (ProtectPrivateVars)
    local *DE_EPAGES::RemoteSearch::API::Attributes::SearchIndex::_getPublicSuggestURLFromConfig = sub {
        return 'http://localhost:81/rs/';
    };
    my $URL = $self->searchIndex->get( 'PublicSuggestURL', GetPKeyLanguageByCode('de') );
    my ( $HostAndPath, $QueryString ) = split( /\?/, $URL );
    like( $HostAndPath, qr|http://localhost:81/rs/product/Store/.*/suggest/jsonp|x, 'host and path match configuration' );
    is_deeply(
        ParseQueryString( $QueryString, 'utf-8' ),
        {
            lang         => ['de'],
        },
        'localized query params match'
    );
}

sub TestPublicSuggestURL : Test {
    my $self = shift;

    # mock configuration.
    ## no critic (ProtectPrivateVars)
    local *DE_EPAGES::RemoteSearch::API::Attributes::SearchIndex::_getPublicSuggestURLFromConfig = sub {
        return;
    };
    my $URL = $self->searchIndex->get( 'PublicSuggestURL', GetPKeyLanguageByCode('de') );

    # skip questionmark.
    $URL = substr( $URL, 1 );
    is_deeply(
        ParseQueryString( $URL, 'utf-8' ),
        {
            'ObjectID'   => [ $self->shop->id ],
            'ViewAction' => ['JSONViewRemoteSuggestProducts'],
        },
        'localized query params match'
    );
}

sub TestIndexStatusString : Test {
    my $self = shift;
    is($self->searchIndex->indexStatusString(), $self->searchIndex->get('IndexStatusString'), 'index status string');
}

sub TestInitializeIndex : Test(no_plan) {
    my $self    = shift;
    my $mockObj = _mockRemoteIndexClient();

    # insert remoteIndex mock. needs to stop perl from complaining.
    no warnings; ## no critic (ProhibitNoWarnings)
    local *DE_EPAGES::RemoteSearch::API::Object::SearchIndex::remoteIndex = sub { return $mockObj; };
    use warnings;
    $self->searchIndex->initialize();
    is( IndexStatusString( $self->searchIndex->get('IndexStatus') ), INDEXSTATUS_PENDING,  'pending index status' );
    is( $self->searchIndex->get('UpdateTaskURL'),                    'http://a.b.c/', 'saved task url' );
}

sub TestInitializeIndexWithFailure : Test(no_plan) {
    my $self    = shift;

    # insert remoteIndex mock. needs to stop perl from complaining.
    no warnings; ## no critic (ProhibitNoWarnings)
    local *DE_EPAGES::RemoteSearch::API::Object::SearchIndex::remoteIndex = sub { Error('communication failure'); };
    use warnings;

    eval {
        $self->searchIndex->initialize();
        fail('should throw an error');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'communication failure', 'escalating error');
        is ($self->searchIndex->indexStatusString, INDEXSTATUS_ERROR, 'index has error status');
    }
}

sub TestIsActive : Test(no_plan) {
    my $self = shift;
    $self->shop->set( { 'RemoteSearchEnabled' => 0 } );
    is( $self->searchIndex->isActive(), 0, 'inactive search' );
    $self->shop->set( { 'RemoteSearchEnabled' => 1 } );
    is( $self->searchIndex->isActive(), 1, 'active search' );
}

sub TestUsableAtStorefront : Test(no_plan) {
    my $self = shift;
    $self->searchIndex->set( { 'IndexStatus' => IndexStatusID(INDEXSTATUS_UNITIALIZED) } );
    is( $self->searchIndex->usableAtStorefront, 0, 'unitialized search index => unusable' );
    $self->searchIndex->set( { 'IndexStatus' => IndexStatusID(INDEXSTATUS_COMPLETE) } );
    is( $self->searchIndex->usableAtStorefront, 1, 'initialized search index => usable' );
}

sub _mockRemoteIndexClient {

    my $mockTask = DE_EPAGES::RemoteSearch::API::DTO::Task->new( {
            'link'  => { 'href' => 'http://a.b.c/' },
            'state' => 'Pending',
        }
    );

    my %mockMethods = (
        'createOrReplaceIndex' => sub { return $mockTask; },
        'dropIndex'            => sub { return $mockTask; },
        'updateItems'          => sub { return $mockTask; },
        'deleteItems'          => sub { return $mockTask; },
    );

    my $mockObj = Test::MockObject->new();
    $mockObj->set_isa('DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient');
    $mockObj->fake_module( 'DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient', %mockMethods );
    while ( my ( $method, $handler ) = each %mockMethods ) {
        $mockObj->mock( $method, $handler );
    }
    return $mockObj;
}

sub shop {
    return shift->{Shop};
}

sub searchIndex {
    return shift->{SearchIndex};
}

RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);
