use strict;
use Test::More tests => 27;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::SEO::API::SEO qw (GetSEOCheckValues CalculateSEOQuality GetWordCount PerformSEOCheck UpdateSEOQuality);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

## no critic qw( ProhibitLongLines )
sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Test = LoadObjectByPath('Test');
    my $Shop = $Test->insertChild('Shop', {
        Alias => 'TestShop',
        InitialShopWizardStartup => 0
    });
    my $ShopLangID = $Shop->get('LanguageID');

    $Shop->featureMaxValue( 'Pages', 10 ); ## no critic qw(ProhibitMagicNumbers)
    $Shop->featureMaxValue( 'Languages', 2);
    my $LanguageID = GetPKeyLanguageByCode( 'de' );
    $Shop->addLanguage($LanguageID);

    my $StartPage = $Shop->child('Categories');

    # test api
    my $Cat123 = $StartPage->insertChild('Category', { Alias => 'Cat123' });
    # direct
    $Cat123->set( { 'SEOQuality' => 33}, $ShopLangID );
    cmp_ok($Cat123->get('SEOQuality', $ShopLangID), '==', 33, 'set ok');

    # api
    my $hVals = {
        1 => { 'NewSEOQuality' => 55 },
        2 => { 'NewSEOQuality' => 66 },
    };
    UpdateSEOQuality($Cat123, $hVals);
    cmp_ok($Cat123->get('SEOQuality', 1), '==', 55, 'UpdateSEOQuality get by lang ok');
    cmp_ok($Cat123->get('SEOQuality', 2), '==', 66, 'UpdateSEOQuality get by lang ok');

    # SEOQualityAverage
    cmp_ok($Cat123->get('SEOQualityAverage'), '==', 60, 'SEOQualityAverage');

    # start page seo test
    my $hValues = GetSEOCheckValues($Shop, $StartPage, $ShopLangID);
    # in clean shop SEO should be very bad:
    ok(!$hValues->{'Name'}, 'GetSEOCheckValues empty name');
    cmp_ok($hValues->{'Url'}->{'Value'}, 'eq', $StartPage->get('CanonicalURL', $ShopLangID), 'GetSEOCheckValues default URL');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Description'}->{'Value'}, 'GetSEOCheckValues empty desc');
    ok(!$hValues->{'AdvDescription'}->{'Value'}, 'GetSEOCheckValues empty long desc');
    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$Token}->{'RatingClass'}, 'eq', 'red', 'GetSEOCheckValues RatingClass red');
    }
    cmp_ok(CalculateSEOQuality($hValues, $StartPage), '==', 0, 'CalculateSEOQuality returns 0');

    # use article and set some SEO-nice values
    $Shop->set({ 'DomainName' => 'www.mytestshop.com' });
    $Shop->set({ 'Name' => 'MyTestShop' }, $ShopLangID);
    my $Art1 = $StartPage->insertChild('Article', { Alias => 'Article' });
    $Art1->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Description' => 'Curabitur pretium elementum purus, ac tincidunt tellus lobortis id. In quis blandit purus. Quisque quis elit varius est faucibus.',
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum.',
        'URI' => 'top-article-kat1',
    }, $ShopLangID);

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => $Art1->get('SEOQualityAverage') || 0,
        SiteUrl => $Shop->get('WebUrl'),
        Domain => $Shop->get('DomainName'),
    };
    TriggerHook('SEOCheckTests', {
        'Object'        => $Art1,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    my $hLangValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    $hValues->{$ShopLangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };

    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$ShopLangID}->{$Token}->{'Rating'}, '==', 1, 'GetSEOCheckValues Rating 1');
    }
    cmp_ok(CalculateSEOQuality($hValues->{$ShopLangID}, $Art1), '==', 90, 'CalculateSEOQuality returns 90 as image is missing');  ## no critic qw(ProhibitMagicNumbers)

    # check data
    my $hPerformValues = PerformSEOCheck($Art1);
    UpdateSEOQuality($Art1, $hPerformValues);
    cmp_ok($Art1->get('SEOQualityAverage'), '==', 50, 'SEOQualityAverage 2');

    #
    cmp_ok(GetWordCount('w1 - w2 | word3+word4 _ w5-w6'), '==', 7,  'GetWordCount counts only words');

    # abstracted from a customer's text
    cmp_ok(
        GetWordCount(
         'AB 1234 cccccc ddddd Ffff-Tttttt mmm Bbbb Dddddddd X-Xxx Ffff-Tttttt mmm Bbbb Dddddddd. 1 kkkkkkkkkkkkkk Aaaaaaaa fff Vvvvvvvvvv, eeeee mmm Eeeee Sssssssss. Mmmmmmmm: 100% Ppppppppp Mmmm: cc. 11 cc llll, 11 cc bbbb MM 1234 mmmmmm bbbbb Ffff-Tttttt mmm Bbbb Dddddddd ii 1 Fffffffffffffffff.'
        ),
        '==',
        44,     ## no critic qw(ProhibitMagicNumbers)
        'GetWordCount counts only words'
    );

    # title and name similarity (EPG-20601)
    $Art1->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Title' => 'Nice test article - all u ever wanted to know! now FREE shipping',
    }, $ShopLangID);
    $hValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    cmp_ok( $hValues->{'Title'}->{'Rating'}, '==', 1, 'EPG-20601: Be more tolarant, if name is part of the title. Title rating is ' . $hValues->{'Title'}->{'Rating'} . '.' );

    $Art1->set({
        'Name' => 'Wirkstoff: Aloe Vera, Swisa Beauty',
        'Title' => 'Swisa Beauty - Aloe Vera - hautfreundliche Produkte',
    }, $ShopLangID);
    $hValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    cmp_ok( $hValues->{'Title'}->{'Rating'}, '==', 1, 'EPG-20601: Be more tolarant, if name is part of the title. Title rating is ' . $hValues->{'Title'}->{'Rating'} . '.' );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    return 1;
}

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
