#========================================================================================
# Package: DE_EPAGES::Trigger::API::Object::BatchProcessHandler and
#          DE_EPAGES::Trigger::API::BatchProcessor
#
# Test: Testing advanced batch processing by creating sql transactions. A list of product
#       is processed in batches of 3 items in one transaction. First we implement a
#       BatchProcessHandler ProductBatchHand by implementing processItem($item) and
#       overriding processItemList. Then we need some hook function wich use HandleItem(...).
#       Finally we can run our business logic, wich trigger our hook function inside
#       RunBatchProcess(...)
#========================================================================================
use strict;
my $print = 0;    # Verbose output; to get an idea what it does

my ( $callsProcessItem, $callsProcessItemList ) = ( 0, 0 );    # globals used for tests

#########################################################################
#
#   Preparations
#
#########################################################################

# no critic - needed for mocking the enviroment to run test
# don't do this in production code!

## no critic (Modules::ProhibitMultiplePackages)
## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)

#
# ProductBatchHandler - generate some sql transactions
#

package My::API::Object::ProductBatchHandler;
use strict;

use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );

sub processItem {
    my $self = shift;
    my ($product) = @_;

    $callsProcessItem++;

    return 'insert into product VALUES (' . $product->{ProductID} . q#,'ProductData');#; ## no critic (ProhibitVariablesInSQLStatements)
}

sub processItemList {
    my $self     = shift;
    my $products = $self->items();

    $callsProcessItemList++;

    my $sqlTransaction = "START TRANSACTION;\n";
    my @values;
    foreach my $product (@$products) {
        push @values, $self->processItem($product);
    }

    print $sqlTransaction . join( "\n", @values ) . "\nCOMMIT;\n" if $print;

    # You must discard processed items explicitly !
    $self->cleanupItems();
}

#
# ProductBatchHooks
#

package My::Hooks::ProductBatchHooks;
use strict;

use DE_EPAGES::Trigger::API::BatchProcessor qw( RegisterHandler IsCalledInBatchMode HandleItem GetHandler);

sub OnRegisterDummyImportProductsHandler {

    # BatchSize control execution of ProductBatchHandler->processItemList()
    RegisterHandler( 'DummyImportProducts', My::API::Object::ProductBatchHandler->new( { BatchSize => 3 } ) );

}

sub OnAfterUpdateProduct($) {
    my ($ProductID) = @_;

    _printStatus($ProductID) if $print;

    # only queue porducts comming from RunBatchProcess
    if ( IsCalledInBatchMode('DummyImportProducts') ) {
        HandleItem( 'DummyImportProducts', 'My::API::Object::ProductBatchHandler', { ProductID => $ProductID } );
    }
    else {

        # handle products which are not associated with 'DummyImportProducts'
    }
}

sub _printStatus() {
    my ($ProductID) = @_;

    my $Handler = GetHandler( 'DummyImportProducts', 'My::API::Object::ProductBatchHandler' );
    print 'add Product ' . $ProductID . ' QueueSize: ' . $Handler->countItems() . "\n";
}

package My::UI::BusinessLogic;

use strict;
use Test::More;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Trigger::API::BatchProcessor qw( RunBatchProcess IsCalledInBatchMode GetHandler );

plan( tests => 7 );

#########################################################################
#
#   Begin Tests
#
#########################################################################

#
#   Mocking TriggerHook
#

# Mocking TriggerHook - BatchProcessor register BatchHandler with DE_EPAGES::Trigger::API::Trigger::TriggerHook()
# in the database. This is not part of the test and is mocked.

no warnings qw(redefine); ## no critic (ProhibitNoWarnings)

sub DE_EPAGES::Trigger::API::BatchProcessor::TriggerHook {

    My::Hooks::ProductBatchHooks::OnRegisterDummyImportProductsHandler();

}

use warnings;

#
# BusinessLogic  - CSV import of products
#

# We emulate a csv import like Product::Scripts::ProductExportImport.pl

sub DummyImportProducts {
    my $Handler;

    # Executes your code, which triggers a hook for adding items for BatchHandler (see DummyImportProduct).
    # After execution of your code BatchHandler->proccessItem() is called automatically

    ok( !IsCalledInBatchMode('DummyImportProducts'), 'outside BatchMode correctly detected' );

    RunBatchProcess(
        'DummyImportProducts',
        sub {
            ok( IsCalledInBatchMode('DummyImportProducts'), 'inside BatchMode correctly detected' );

            foreach my $ProductID ( 1 .. 8 ) {
                DummyImportProduct($ProductID);
            }

            $Handler = GetHandler( 'DummyImportProducts', 'My::API::Object::ProductBatchHandler' );
            is( $callsProcessItemList,  2, 'processItemList is called two times' );
            is( $Handler->countItems(), 2, '8 items - 6 items processed = 2 items left' );
        }
    );
    ok( $Handler->countItems() == 0, 'last two items are consumed and queue cleared' );
    is( $callsProcessItemList, 3, 'processItemList was called three times' );
    is( $callsProcessItem,     8, 'processItem was called eight times' );

}

sub DummyImportProduct {
    my ($ProductID) = @_;

    # do CSV import for current Object

    # mocking TriggerHook('OnAfterUpdateProduct',...)
    My::Hooks::ProductBatchHooks::OnAfterUpdateProduct($ProductID);
    return;
}

RunScript( Sub => \&DummyImportProducts );
